﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using ChartSample;
using Microsoft.Phone.Reactive;
using Syncfusion.SfChart.XForms;
using Syncfusion.SfChart.XForms.WinPhone;
using Syncfusion.UI.Xaml.Charts;
using Xamarin.Forms;
using Xamarin.Forms.Platform.WinPhone;
using System.Windows.Media;
using System.Windows.Markup;
[assembly: ExportRenderer(typeof(CustomChart), typeof( ChartSample.WinPhone.CustomRenderer))]

namespace ChartSample.WinPhone
{
    public class CustomRenderer : SfChartRenderer
    {
        private CustomChart chart;

        protected override void OnElementChanged(ElementChangedEventArgs<Syncfusion.SfChart.XForms.SfChart> e)
        {
            base.OnElementChanged(e);
            if (Control == null) return;
            StringBuilder sb = new StringBuilder();
            chart = e.NewElement as CustomChart;

            var nativeAxis = SfChartRenderer.GetNativeObject(typeof(ChartLabelStyle), chart.SecondaryAxis.LabelStyle) as Syncfusion.UI.Xaml.Charts.ChartAxis;

            var textBoxString = "";
            var borderString = "";

            SetDataTemplate(ref borderString, ref textBoxString, chart.SecondaryAxis.LabelStyle.BackgroundColor,
                 chart.SecondaryAxis.LabelStyle.BorderColor, chart.SecondaryAxis.LabelStyle.BorderThickness,
                 chart.SecondaryAxis.LabelStyle.TextColor, chart.SecondaryAxis.LabelStyle.Font, chart.SecondaryAxis.LabelStyle.Margin);
            sb.Clear();
            sb.Append("<DataTemplate xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\"");
            sb.Append(" xmlns:x=\"http://schemas.microsoft.com/winfx/2006/xaml\"");
            sb.Append(" xmlns:valueConverter='clr-namespace:ChartSample.WinPhone;assembly=ChartSample.WinPhone'>");
            sb.Append("<Border " + borderString + " >");
            sb.Append("<Border.Resources>");
            sb.Append("<valueConverter:LabelConverter x:Key='labelConverter'/>");
            sb.Append("</Border.Resources>");
            sb.Append("<TextBlock Text='{Binding Converter={StaticResource labelConverter}}\' " + textBoxString + "/>");
            sb.Append("</Border></DataTemplate>");

            Control.SecondaryAxis.LabelTemplate = (System.Windows.DataTemplate)XamlReader.Load(sb.ToString());
        }

        

        internal static void SetDataTemplate(ref string borderString, ref string textBoxString, Xamarin.Forms.Color backgroundColor, Xamarin.Forms.Color borderColor, Xamarin.Forms.Thickness borderThickness, Xamarin.Forms.Color textColor, Font textFont, Xamarin.Forms.Thickness textMargin)
        {
            if (backgroundColor != Xamarin.Forms.Color.Default)
            {
                borderString += " Background=\"" + new SolidColorBrush(SfChartRenderer.GetWindowsColor(backgroundColor)).Color.ToString() + "\"";
            }
            if (borderColor != Xamarin.Forms.Color.Default)
            {
                borderString += " BorderBrush=\"" + new SolidColorBrush(SfChartRenderer.GetWindowsColor(borderColor)).Color.ToString() + "\"";
            }
            borderString += " BorderThickness=\"" + borderThickness.Top + " " + borderThickness.Right + " " + borderThickness.Bottom + " " + borderThickness.Left + "\"";

            if (textColor != Xamarin.Forms.Color.Default)
            {
                textBoxString += " Foreground=\"" + new SolidColorBrush(SfChartRenderer.GetWindowsColor(textColor)).Color.ToString() + "\"";
            }
            if (textFont.FontSize != 0)
            {
                textBoxString += " Fontsize =\"" + textFont.FontSize.ToString() + "\"";
            }
            if (textFont.FontFamily != null)
            {
                textBoxString += " FontFamily =\"" + textFont.FontFamily + "\"";
            }

            if (textFont.FontAttributes == Xamarin.Forms.FontAttributes.Bold)
                textBoxString += " FontStyle =\"Oblique\"";
            else if (textFont.FontAttributes == Xamarin.Forms.FontAttributes.None)
                textBoxString += " FontStyle =\"Normal\"";
            else
                textBoxString += " FontStyle =\"" + textFont.FontAttributes + "\"";

            textBoxString += " Margin =\"" + textMargin.Top + " " + textMargin.Right + " " + textMargin.Bottom + " " + textMargin.Left + "\"";

        }
    }

    public class LabelConverter : System.Windows.Data.IValueConverter
    {

        static int preMonth = -1;

        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {

            Syncfusion.UI.Xaml.Charts.ChartAxisLabel label = value as Syncfusion.UI.Xaml.Charts.ChartAxisLabel;

            label.Position = label.Position + 0.5;

            //DateTime dateLabel = DateTime.Parse((string)label.LabelContent);

            //int currMonth = dateLabel.Month;
            //if (preMonth != currMonth)
            //{
            //    label.LabelContent = dateLabel.ToString("MMM");
            //    preMonth = currMonth;
            //}
            //else
            //    label.LabelContent = dateLabel.ToString("dd");

            return label.LabelContent;

        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
